@@ -1,13 +1,14 @@
 from lxml import etree as ET
 from lxml.etree import Element
 import os, shutil
-import logging, time
+import logging
 import tempfile
-from . import static, utils, generate, core
+from . import utils, generate, core
 from . import build as builder
 from pathlib import Path
 import sys
 from typing import Optional
+import webbrowser
 
 log = logging.getLogger('ptxlogger')
 
@@ -137,10 +138,12 @@ class Project():
             for target_element in self.xml_element().xpath("targets/target")
         ]
 
-    def target_names(self):
+    def target_names(self, *args):
+        # Optional arguments are formats: returns list of targets that have that format.
         names = []
         for target in self.targets():
-            names.append(target.name())
+            if not args or target.format() in args:
+                names.append(target.name())
         return names
 
     def print_target_names(self):
@@ -157,7 +160,7 @@ class Project():
         else:
             return None
 
-    def view(self,target_name:str,access:str,port:int,watch:bool=False):
+    def view(self,target_name:str,access:str,port:int,watch:bool=False, no_launch:bool=False):
         target = self.target(target_name)
         directory = target.output_dir()
         if watch:
@@ -169,13 +172,24 @@ class Project():
             log.error(f"Run `pretext view {target.name()} -b` to build your project before viewing.")
             return
         watch_callback=lambda:self.build(target_name)
-        utils.run_server(directory,access,port,watch_directory,watch_callback)
+        if utils.cocalc_project_id() is not None:
+            if target.format() in ['html', 'pdf']:
+                utils.run_server(directory,access,port,watch_directory,watch_callback,no_launch)
+            else:
+                log.info(f"Output can be viewed by navigating to {directory}")
+        elif target.format() == 'html':
+            utils.run_server(directory,access,port,watch_directory,watch_callback,no_launch)
+        else:
+            outputfiles = list(Path(directory).glob("*.*"))
+            if len(outputfiles) > 1 or no_launch:
+                log.info(f"Output can be viewed by navigating to {directory}")
+            else:
+                outputfile = outputfiles[0]
+                log.info(f"Attempting to open {outputfile} using default viewer for {target.format()} files.")
+                webbrowser.open(outputfile)
 
     def build(self,target_name,clean=False):
         target = self.target(target_name)
-        if target is None:
-            log.error(f"Target {target_name} not found.")
-            return
         # Check for xml syntax errors and quit if xml invalid:
         if not self.xml_source_is_valid(target_name):
             return
@@ -219,10 +233,21 @@ class Project():
                 if custom_xsl is None:
                     raise Exception("Must specify custom XSL for custom build.")
                 builder.custom(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl,target.output_filename())
+            elif target.format()=='epub':
+                builder.epub(target.source(),target.publication(),target.output_dir(),target.stringparams())
+            elif target.format()=='kindle':
+                builder.kindle(target.source(),target.publication(),target.output_dir(),target.stringparams())
+            elif target.format()=='braille' or target.format()=='braille-emboss':
+                builder.braille(target.source(),target.publication(),target.output_dir(),target.stringparams(),page_format="emboss")
+            elif target.format()=='braille-electronic':
+                builder.braille(target.source(),target.publication(),target.output_dir(),target.stringparams(),page_format="electronic")
+            else:
+                log.critical(f'The build format {target.format()} is not supported.')
         except Exception as e:
             log.critical(
                 f"A fatal error has occurred:\n {e} \nFor more info, run pretext with `-v debug`")
-            log.debug(f"Critical error info:\n****\n", exc_info=True)
+            log.debug(f"Exception info:\n##################\n", exc_info=True)
+            log.info('##################')
             sys.exit(f"Failed to build pretext target {target.format()}.  Exiting...")
         # build was successful
         log.info(f"\nSuccess! Run `pretext view {target.name()}` to see the results.\n")
@@ -285,7 +310,7 @@ class Project():
             log.error("Visit https://github.com/git-guides/install-git for assistance.")
             return
         target = self.target(target_name)
-        if target.format() != "html":
+        if target.format() != "html": #redundent for CLI
             log.error("Only HTML format targets are supported.")
             return
         try:
